@@ -1,4 +1,4 @@
-/* $Id: timeval-test.c 305 2005-08-12 19:09:50Z lennart $ */
+/* $Id: timeval-test.c 879 2005-10-26 21:27:40Z lennart $ */
 
 /***
   This file is part of avahi.
@@ -24,9 +24,11 @@
 #endif
 
 #include <stdio.h>
+
+#include "gccmacro.h"
 #include "timeval.h"
 
-int main(int argc, char *argv[]) {
+int main(AVAHI_GCC_UNUSED int argc, AVAHI_GCC_UNUSED char *argv[]) {
 
     struct timeval a = { 5, 5 }, b;
 
@@ -38,4 +40,6 @@ int main(int argc, char *argv[]) {
     printf("%li.%li\n", a.tv_sec, a.tv_usec);
 
     printf("%lli\n", avahi_timeval_diff(&a, &b));
+
+    return 0;
 }
