@@ -1,7 +1,7 @@
 #ifndef foobrowsehfoo
 #define foobrowsehfoo
 
-/* $Id: browse.h 386 2005-08-21 20:03:50Z lennart $ */
+/* $Id: browse.h 804 2005-10-17 22:28:01Z lennart $ */
 
 /***
   This file is part of avahi.
@@ -23,19 +23,40 @@
 ***/
 
 #include <avahi-common/llist.h>
+
 #include "core.h"
 #include "timeeventq.h"
-#include "server.h"
+#include "internal.h"
+#include "dns.h"
+#include "lookup.h"
+
+typedef struct AvahiSRBLookup AvahiSRBLookup;
+
+struct AvahiSRecordBrowser {
+    AVAHI_LLIST_FIELDS(AvahiSRecordBrowser, browser);
+    int dead;
+    AvahiServer *server;
+
+    AvahiKey *key;
+    AvahiIfIndex interface;
+    AvahiProtocol protocol;
+    AvahiLookupFlags flags;
+    
+    AvahiTimeEvent *defer_time_event;
+
+    AvahiSRecordBrowserCallback callback;
+    void* userdata;
+
+    /* Lookup data */
+    AVAHI_LLIST_HEAD(AvahiSRBLookup, lookups);
+    unsigned n_lookups;
+
+    AvahiSRBLookup *root_lookup;
+};
 
 void avahi_browser_cleanup(AvahiServer *server);
-void avahi_browser_notify(AvahiServer *s, AvahiInterface *i, AvahiRecord *record, AvahiBrowserEvent event);
-
-int avahi_is_subscribed(AvahiServer *s, AvahiInterface *i, AvahiKey *k);
 
 void avahi_s_record_browser_destroy(AvahiSRecordBrowser *b);
-
 void avahi_s_record_browser_restart(AvahiSRecordBrowser *b);
 
-void avahi_browser_new_interface(AvahiServer*s, AvahiInterface *i);
-
 #endif
