@@ -1,7 +1,7 @@
 #ifndef foonetlinkhfoo
 #define foonetlinkhfoo
 
-/* $Id: netlink.h 314 2005-08-14 02:33:41Z lennart $ */
+/* $Id: netlink.h 801 2005-10-17 20:20:53Z lennart $ */
 
 /***
   This file is part of avahi.
@@ -24,7 +24,10 @@
 
 #include <sys/socket.h>
 #include <asm/types.h>
+#include <inttypes.h>
+
 #include <linux/netlink.h>
+#include <linux/rtnetlink.h>
 
 #include <avahi-common/watch.h>
 
