@@ -1,4 +1,4 @@
-/* $Id: rr.c 308 2005-08-13 21:25:09Z lennart $ */
+/* $Id: rr.c 1209 2006-05-04 21:50:44Z lennart $ */
 
 /***
   This file is part of avahi.
@@ -32,11 +32,14 @@
 
 #include <avahi-common/domain.h>
 #include <avahi-common/malloc.h>
+#include <avahi-common/defs.h>
 
 #include "rr.h"
 #include "log.h"
 #include "util.h"
 #include "hashmap.h"
+#include "domain-util.h"
+#include "rr-util.h"
 
 AvahiKey *avahi_key_new(const char *name, uint16_t class, uint16_t type) {
     AvahiKey *k;
@@ -47,7 +50,7 @@ AvahiKey *avahi_key_new(const char *name, uint16_t class, uint16_t type) {
         return NULL;
     }
     
-    if (!(k->name = avahi_normalize_name(name))) {
+    if (!(k->name = avahi_normalize_name_strdup(name))) {
         avahi_log_error("avahi_normalize_name() failed.");
         avahi_free(k);
         return NULL;
@@ -60,6 +63,18 @@ AvahiKey *avahi_key_new(const char *name, uint16_t class, uint16_t type) {
     return k;
 }
 
+AvahiKey *avahi_key_new_cname(AvahiKey *key) {
+    assert(key);
+
+    if (key->clazz != AVAHI_DNS_CLASS_IN)
+        return NULL;
+
+    if (key->type == AVAHI_DNS_TYPE_CNAME)
+        return NULL;
+
+    return avahi_key_new(key->name, key->clazz, AVAHI_DNS_TYPE_CNAME);
+}
+
 AvahiKey *avahi_key_ref(AvahiKey *k) {
     assert(k);
     assert(k->ref >= 1);
@@ -142,6 +157,7 @@ void avahi_record_unref(AvahiRecord *r) {
 
             case AVAHI_DNS_TYPE_PTR:
             case AVAHI_DNS_TYPE_CNAME:
+            case AVAHI_DNS_TYPE_NS:
                 avahi_free(r->data.ptr.name);
                 break;
 
@@ -199,24 +215,40 @@ const char *avahi_dns_type_to_string(uint16_t type) {
             return "SRV";
         case AVAHI_DNS_TYPE_ANY:
             return "ANY";
+        case AVAHI_DNS_TYPE_SOA:
+            return "SOA";
+        case AVAHI_DNS_TYPE_NS:
+            return "NS";
         default:
             return NULL;
     }
 }
 
 char *avahi_key_to_string(const AvahiKey *k) {
+    char class[16], type[16];
+    const char *c, *t;
+    
     assert(k);
     assert(k->ref >= 1);
+
+    /* According to RFC3597 */
     
-    return avahi_strdup_printf("%s\t%s\t%s",
-                               k->name,
-                               avahi_dns_class_to_string(k->clazz),
-                               avahi_dns_type_to_string(k->type));
+    if (!(c = avahi_dns_class_to_string(k->clazz))) {
+        snprintf(class, sizeof(class), "CLASS%u", k->clazz);
+        c = class;
+    }
+
+    if (!(t = avahi_dns_type_to_string(k->type))) {
+        snprintf(type, sizeof(type), "TYPE%u", k->type);
+        t = type;
+    }
+    
+    return avahi_strdup_printf("%s\t%s\t%s", k->name, c, t);
 }
 
 char *avahi_record_to_string(const AvahiRecord *r) {
     char *p, *s;
-    char buf[257], *t = NULL, *d = NULL;
+    char buf[1024], *t = NULL, *d = NULL;
 
     assert(r);
     assert(r->ref >= 1);
@@ -231,7 +263,8 @@ char *avahi_record_to_string(const AvahiRecord *r) {
             break;
             
         case AVAHI_DNS_TYPE_PTR:
-        case AVAHI_DNS_TYPE_CNAME :
+        case AVAHI_DNS_TYPE_CNAME:
+        case AVAHI_DNS_TYPE_NS:
 
             t = r->data.ptr.name;
             break;
@@ -254,10 +287,34 @@ char *avahi_record_to_string(const AvahiRecord *r) {
                      r->data.srv.name);
 
             break;
+
+        default: {
+
+            uint8_t *c;
+            uint16_t n;
+            int i;
+            char *e;
+
+            /* According to RFC3597 */
+            
+            snprintf(t = buf, sizeof(buf), "\\# %u", r->data.generic.size);
+
+            e = strchr(t, 0);
+            
+            for (c = r->data.generic.data, n = r->data.generic.size, i = 0;
+                 n > 0 && i < 20;
+                 c ++, n --, i++) {
+
+                sprintf(e, " %02X", *c);
+                e = strchr(e, 0);
+            }
+
+            break;
+        }
     }
 
     p = avahi_key_to_string(r->key);
-    s = avahi_strdup_printf("%s %s ; ttl=%u", p, t ? t : "<unparsable>", r->ttl);
+    s = avahi_strdup_printf("%s %s ; ttl=%u", p, t, r->ttl);
     avahi_free(p);
     avahi_free(d);
     
@@ -271,8 +328,6 @@ int avahi_key_equal(const AvahiKey *a, const AvahiKey *b) {
     if (a == b)
         return 1;
     
-/*     g_message("equal: %p %p", a, b); */
-    
     return avahi_domain_equal(a->name, b->name) &&
         a->type == b->type &&
         a->clazz == b->clazz;
@@ -282,8 +337,6 @@ int avahi_key_pattern_match(const AvahiKey *pattern, const AvahiKey *k) {
     assert(pattern);
     assert(k);
 
-/*     g_message("equal: %p %p", a, b); */
-
     assert(!avahi_key_is_pattern(k));
 
     if (pattern == k)
@@ -316,15 +369,6 @@ static int rdata_equal(const AvahiRecord *a, const AvahiRecord *b) {
     assert(b);
     assert(a->key->type == b->key->type);
 
-/*     t = avahi_record_to_string(a); */
-/*     g_message("comparing %s", t); */
-/*     avahi_free(t); */
-
-/*     t = avahi_record_to_string(b); */
-/*     g_message("and %s", t); */
-/*     avahi_free(t); */
-
-    
     switch (a->key->type) {
         case AVAHI_DNS_TYPE_SRV:
             return
@@ -335,6 +379,7 @@ static int rdata_equal(const AvahiRecord *a, const AvahiRecord *b) {
 
         case AVAHI_DNS_TYPE_PTR:
         case AVAHI_DNS_TYPE_CNAME:
+        case AVAHI_DNS_TYPE_NS:
             return avahi_domain_equal(a->data.ptr.name, b->data.ptr.name);
 
         case AVAHI_DNS_TYPE_HINFO:
@@ -386,6 +431,7 @@ AvahiRecord *avahi_record_copy(AvahiRecord *r) {
     switch (r->key->type) {
         case AVAHI_DNS_TYPE_PTR:
         case AVAHI_DNS_TYPE_CNAME:
+        case AVAHI_DNS_TYPE_NS:
             if (!(copy->data.ptr.name = avahi_strdup(r->data.ptr.name)))
                 goto fail;
             break;
@@ -455,6 +501,7 @@ size_t avahi_record_get_estimate_size(AvahiRecord *r) {
     switch (r->key->type) {
         case AVAHI_DNS_TYPE_PTR:
         case AVAHI_DNS_TYPE_CNAME:
+        case AVAHI_DNS_TYPE_NS:
             n += strlen(r->data.ptr.name) + 1;
             break;
 
@@ -530,6 +577,7 @@ int avahi_record_lexicographical_compare(AvahiRecord *a, AvahiRecord *b) {
 
         case AVAHI_DNS_TYPE_PTR:
         case AVAHI_DNS_TYPE_CNAME:
+        case AVAHI_DNS_TYPE_NS:
             return avahi_binary_domain_cmp(a->data.ptr.name, b->data.ptr.name);
 
         case AVAHI_DNS_TYPE_SRV: {
@@ -627,6 +675,7 @@ int avahi_record_is_valid(AvahiRecord *r) {
 
         case AVAHI_DNS_TYPE_PTR:
         case AVAHI_DNS_TYPE_CNAME:
+        case AVAHI_DNS_TYPE_NS:
             return avahi_is_valid_domain_name(r->data.ptr.name);
 
         case AVAHI_DNS_TYPE_SRV:
@@ -636,14 +685,13 @@ int avahi_record_is_valid(AvahiRecord *r) {
             return
                 strlen(r->data.hinfo.os) <= 255 &&
                 strlen(r->data.hinfo.cpu) <= 255;
-
             
         case AVAHI_DNS_TYPE_TXT: {
 
             AvahiStringList *strlst;
 
             for (strlst = r->data.txt.string_list; strlst; strlst = strlst->next)
-                if (strlst->size > 255)
+                if (strlst->size > 255 || strlst->size <= 0)
                     return 0;
 
             return 1;
