@@ -1,7 +1,7 @@
 #ifndef foodbusprotocolhfoo
 #define foodbusprotocolhfoo
 
-/* $Id: dbus-protocol.h 327 2005-08-15 02:42:59Z lennart $ */
+/* $Id: dbus-protocol.h 1197 2006-04-24 01:48:56Z lennart $ */
 
 /***
   This file is part of avahi.
@@ -22,7 +22,7 @@
   USA.
 ***/
 
-int dbus_protocol_setup(const AvahiPoll *poll_api);
+int dbus_protocol_setup(const AvahiPoll *poll_api, int disable_user_service_publishing, int force);
 void dbus_protocol_shutdown(void);
 void dbus_protocol_server_state_changed(AvahiServerState state);
 
