@@ -1,7 +1,7 @@
 #ifndef fooglibwatchhfoo
 #define fooglibwatchhfoo
 
-/* $Id: glib-watch.h 1151 2006-02-20 16:21:29Z lennart $ */
+/* $Id: glib-watch.h 1255 2006-08-22 00:02:24Z lennart $ */
 
 /***
   This file is part of avahi.
@@ -24,6 +24,9 @@
 
 /** \file glib-watch.h GLib main loop adapter */
 
+/** \example glib-integration.c Example of how to integrate
+ * avahi use with GLIB/GTK applications */
+
 #include <glib.h>
 
 #include <avahi-common/cdecl.h>
@@ -37,7 +40,7 @@ typedef struct AvahiGLibPoll AvahiGLibPoll;
 /** Create a new GLib main loop adapter attached to the specified
  context. If context is NULL, the default main loop context is
  used. You can attach as many AvahiGLibPoll objects to the same context
- as you want. priority takes on of GLib's G_PRIORITY constants. */
+ as you want. priority takes one of GLib's G_PRIORITY constants. */
 AvahiGLibPoll *avahi_glib_poll_new(GMainContext *context, gint priority);
 
 /** Free  GLib main loop adapter */
